package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysLocaleResourceService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.BeanCopyTool;
import cn.estsh.i3plus.platform.common.tool.JsonUtilTool;
import cn.estsh.i3plus.platform.common.tool.SerializeTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.model.platform.MissResourceModel;
import cn.estsh.i3plus.pojo.platform.bean.SysLocaleLanguage;
import cn.estsh.i3plus.pojo.platform.bean.SysLocaleResource;
import cn.estsh.i3plus.pojo.platform.bean.SysMenu;
import cn.estsh.i3plus.pojo.platform.repository.SysLocaleLanguageRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysLocaleResourceRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static cn.estsh.i3plus.platform.common.util.CommonConstWords.RESOURCE_PREFIX_MENU;

/**
 * @Description : 系统资源
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2018-11-12 14:46
 * @Modify:
 **/
@Service
public class SysLocaleResourceService implements ISysLocaleResourceService {
    private static final Logger LOGGER = LoggerFactory.getLogger(SysLocaleLanguageService.class);

    @Autowired
    private SysLocaleResourceRepository sysLocaleResourceRDao;

    @Autowired
    private SysLocaleLanguageRepository sysLocaleLanguageRDao;

    @Resource(name = CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;


    @Override
    @ApiOperation(value = "新增系统资源")
    public SysLocaleResource insertSysLocaleResource(SysLocaleResource sysLocaleResource) {
        LOGGER.info("系统资源 SYS_LOCALE_RESOURCE SysLocaleResource：{}", sysLocaleResource);
        int count = sysLocaleResourceRDao.findByPropertyCount(
                new String[]{"languageCode", "resourceKey", "softType"},
                new Object[]{sysLocaleResource.getLanguageCode(), sysLocaleResource.getResourceKey(), sysLocaleResource.getSoftType()});
        if (count > 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("语言资源已存在")
                    .setErrorSolution("请重新输入语言代码与资源键")
                    .build();
        }

        // 设置语言名称冗余
        SysLocaleLanguage sysLocaleLanguage = sysLocaleLanguageRDao.getByProperty("languageCode",
                sysLocaleResource.getLanguageCode());
        ValidatorBean.checkNotNull(sysLocaleLanguage, "选中的语言不存在");
        sysLocaleResource.setLanguageNameRdd(sysLocaleLanguage.getLanguageName());

        return sysLocaleResourceRDao.insert(sysLocaleResource);
    }


    @Override
    public List<SysLocaleResource> saveSysLocaleResource(List<SysLocaleResource> list) {
        LOGGER.info("系统资源 SYS_LOCALE_RESOURCE list：{}", list);
        for (SysLocaleResource sysLocaleResource : list) {
            sysLocaleResourceRDao.insert(sysLocaleResource);
        }
        return list;
    }

    @Override
    @ApiOperation(value = "修改系统资源", notes = "修改系统资源")
    public void updateSysLocaleResource(SysLocaleResource sysLocaleResource) {
        SysLocaleResource originSysLocaleResource = sysLocaleResourceRDao.getById(sysLocaleResource.getId());
        LOGGER.info("系统资源 SYS_LOCALE_RESOURCE SysLocaleResource：{}", sysLocaleResource);
        String hqlPack = CoreHqlPack.packHqlSysLocaleResourceKey(sysLocaleResource);
        int count = sysLocaleResourceRDao.findByHqlWhereCount(hqlPack);
        if (count > 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("语言资源已存在")
                    .setErrorSolution("请重新输入语言代码与资源键")
                    .build();
        }

        // 设置语言名称冗余
        SysLocaleLanguage sysLocaleLanguage = sysLocaleLanguageRDao.getByProperty("languageCode",
                sysLocaleResource.getLanguageCode());
        ValidatorBean.checkNotNull(sysLocaleLanguage, "选中的语言不存在");
        sysLocaleResource.setLanguageNameRdd(sysLocaleLanguage.getLanguageName());

        BeanCopyTool.beanCopy(sysLocaleResource, originSysLocaleResource);

        sysLocaleResourceRDao.update(originSysLocaleResource);
    }

    @Override
    @ApiOperation(value = "根据id删除系统资源", notes = "根据id删除系统资源")
    public void deleteSysLocaleResourceById(Long id) {
        LOGGER.info("系统资源 SYS_LOCALE_RESOURCE id：{}", id);
        sysLocaleResourceRDao.deleteById(id);
    }

    @Override
    @ApiOperation(value = "查询全部系统资源", notes = "查询全部系统资源")
    public List listSysLocaleResource() {
        LOGGER.info("系统资源 SYS_LOCALE_RESOURCE list");
        return sysLocaleResourceRDao.list();
    }

    @Override
    @ApiOperation(value = "系统资源复杂查询，分页，排序", notes = "系统资源复杂查询，分页，排序")
    public ListPager querySysLocaleResourceByPager(SysLocaleResource sysLocaleResource, Pager pager) {
        LOGGER.info("系统资源 SYS_LOCALE_RESOURCE SysLocaleResource：{}，Pager：{}", sysLocaleResource, pager);
        DdlPackBean packBean = CoreHqlPack.packHqlSysLocaleResource(sysLocaleResource);
        pager = PagerHelper.getPager(pager, sysLocaleResourceRDao.findByHqlWhereCount(packBean));
        return new ListPager(sysLocaleResourceRDao.findByHqlWherePage(packBean, pager), pager);
    }

    @Override
    @ApiOperation(value = "根据id批量删除", notes = "根据id批量删除")
    public void deleteSysLocaleResourceByIds(Long[] ids) {
        LOGGER.info("系统资源 SYS_LOCALE_RESOURCE ids：{}", ids);
        sysLocaleResourceRDao.deleteByIds(ids);
    }

    @Override
    @ApiOperation(value = "批量添加")
    public void insertSysLocaleResourceList(List<SysLocaleResource> sysLocaleResourcesList) {
        // 添加校验
        SysLocaleResource sysLocaleResource = sysLocaleResourcesList.get(0);
        LOGGER.info("系统资源 SYS_LOCALE_RESOURCE SysLocaleResource：{}", sysLocaleResource);
        int count = sysLocaleResourceRDao.findByPropertyCount(new String[]{"languageCode", "resourceKey"},
                new String[]{sysLocaleResource.getLanguageCode(), sysLocaleResource.getResourceKey()});
        if (count > 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("语言资源已存在")
                    .setErrorSolution("请重新输入语言代码与资源键")
                    .build();
        }

        // 设置语言名称冗余
        SysLocaleLanguage sysLocaleLanguage = sysLocaleLanguageRDao.getByProperty("languageCode",
                sysLocaleResource.getLanguageCode());
        ValidatorBean.checkNotNull(sysLocaleLanguage, "选中的语言不存在");
        sysLocaleResource.setLanguageNameRdd(sysLocaleLanguage.getLanguageName());

        sysLocaleResourceRDao.saveAll(sysLocaleResourcesList);
    }

    @Override
    @ApiOperation(value = "根据语言code和资源key查询资源")
    public SysLocaleResource getSysLocaleResourceByLanguageCodeAndResKey(String languageCode, String resKey) {
        return sysLocaleResourceRDao.getByProperty(new String[]{"languageCode", "resourceKey"}, new String[]{languageCode, resKey});
    }


    @Override
    public List<SysLocaleResource> findSysLocaleResourceByResKey(String resKey) {
        return sysLocaleResourceRDao.findByProperty(new String[]{"resourceKey"}, new String[]{resKey});
    }

    @Override
    public List<SysLocaleResource> findSysLocaleResource(SysLocaleResource sysLocaleResource) {
        return sysLocaleResourceRDao.findByHqlWhere(CoreHqlPack.packHqlSysLocaleResource(sysLocaleResource));
    }

    @Override
    public SysLocaleResource getSysLocaleResourceById(Long id) {
        return sysLocaleResourceRDao.getById(id);
    }

    @Override
    public List<SysMenu> tranSysMenu(List<SysMenu> menuList) {
        if (menuList != null && !menuList.isEmpty()) {
            String languageCode = (String) AuthUtil.getSessionLanguage();
            menuList = (List<SysMenu>) SerializeTool.copyObject(menuList);

            String redisLanguageKey = CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE_MENU + ":" + languageCode;
            Map menuResourceMap = redisRes.getHashMap(redisLanguageKey);

            if (menuResourceMap != null && menuResourceMap.size() > 0) {
                for (SysMenu menu : menuList) {
                    if (menu != null) {
                        if (StringUtils.isNotBlank(menu.getName())) {
                            String menuName = (String) menuResourceMap.get(RESOURCE_PREFIX_MENU + menu.getName());
                            menu.setName(StringUtils.isNotBlank(menuName) ? menuName : menu.getName());
                        }

                        if (StringUtils.isNotBlank(menu.getParentNameRdd())) {
                            String parentNameRdd = (String) menuResourceMap.get(RESOURCE_PREFIX_MENU + menu.getParentNameRdd());
                            menu.setParentNameRdd(StringUtils.isNotBlank(parentNameRdd) ? parentNameRdd : menu.getParentNameRdd());
                        }
                    }

                }
            }
        }
        return menuList;
    }

    @Override
    public void deleteSysLocaleResourceAll() {
        sysLocaleResourceRDao.deleteAllInBatch();
    }

    @Override
    public List<MissResourceModel> listMissResourceModel() {
        List<Object> listStr = redisRes.scan(CommonConstWords.REDIS_PREFIX_CACHE_NO_RES + "*");
        MissResourceModel missResourceModel = null;
        List<MissResourceModel> missResourceModelList = new ArrayList<>(listStr.size());

        // 更新状态
        for (Object missResourceStr : listStr) {
            if (missResourceStr == null || StringUtils.isBlank(missResourceStr.toString())) {
                continue;
            }
            missResourceModel = JsonUtilTool.decode(missResourceStr.toString(), MissResourceModel.class);
            // 转换失败
            if(missResourceModel==null){
                continue;
            }
            missResourceModelList.add(missResourceModel);
        }
        return missResourceModelList;
    }

    @Override
    public MissResourceModel saveMissResourceModel(MissResourceModel missResourceModel) {
        redisRes.putObject(missResourceModel.getKey(), JSON.toJSONString(missResourceModel), -1);
        return missResourceModel;
    }

    @Override
    public void updateMissResourceModelIsValidByKeys(String[] keys, Integer valid) {
        List<Object> missResourceStrList = redisRes.findObject(Arrays.asList(keys));
        MissResourceModel missResourceModel = null;
        List<MissResourceModel> missResourceModelList = new ArrayList<>(missResourceStrList.size());

        // 更新状态
        for (Object missResourceStr : missResourceStrList) {
            if (missResourceStr == null || StringUtils.isBlank(missResourceStr.toString())) {
                continue;
            }
            missResourceModel = JsonUtilTool.decode(missResourceStr.toString(), MissResourceModel.class);
            missResourceModel.setIsValid(valid);
            missResourceModelList.add(missResourceModel);
        }

        // 存入redis
        for (MissResourceModel resourceModel : missResourceModelList) {
            redisRes.putObject(resourceModel.getKey(), JsonUtilTool.encode(resourceModel));
        }
    }

    @Override
    public void doCleanInValidMissResource() {
        List<Object> listStr = redisRes.scan(CommonConstWords.REDIS_PREFIX_CACHE_NO_RES + "*");
        MissResourceModel missResourceModel = null;
        List<String> cleanKeys = new ArrayList<>(listStr.size());

        // 更新状态
        for (Object missResourceStr : listStr) {
            if (missResourceStr ==null || StringUtils.isBlank(missResourceStr.toString())) {
                continue;
            }
            missResourceModel = JsonUtilTool.decode(missResourceStr.toString(), MissResourceModel.class);

            if (missResourceModel != null && missResourceModel.getIsValid() != null
                    && CommonEnumUtil.IS_VAILD.INVAILD.getValue() == missResourceModel.getIsValid()) {
                cleanKeys.add(missResourceModel.getKey());
            }
        }
        redisRes.deleteKey(cleanKeys.toArray(new String[cleanKeys.size()]));
    }

    @Override
    public void doConversionMissResourceByKeys(String[] keys) {
        List<Object> missResourceStrList = redisRes.findObject(Arrays.asList(keys));
        MissResourceModel missResourceModel = null;
        List<MissResourceModel> missResourceModelList = new ArrayList<>(missResourceStrList.size());
        Map<String, String> langCodeMap = sysLocaleLanguageRDao.list()
                .stream()
                .collect(
                        Collectors.toMap(
                                SysLocaleLanguage::getLanguageCode,
                                SysLocaleLanguage::getLanguageName
                        )
                );

        // 类型转换
        for (Object missResourceStr : missResourceStrList) {
            if (missResourceStr == null || StringUtils.isBlank(missResourceStr.toString())) {
                continue;
            }
            missResourceModel = JsonUtilTool.decode(missResourceStr.toString(), MissResourceModel.class);
            missResourceModelList.add(missResourceModel);
        }

        SysLocaleResource sysLocaleResource;
        for (MissResourceModel resourceModel : missResourceModelList) {
            sysLocaleResource = new SysLocaleResource();
            sysLocaleResource.setSoftType(resourceModel.getSoftType());
            sysLocaleResource.setLanguageCode(resourceModel.getLanguageCode());
            sysLocaleResource.setLanguageNameRdd(langCodeMap.get(resourceModel.getLanguageCode()));
            sysLocaleResource.setResourceType(resourceModel.getResourceType());
            sysLocaleResource.setResourceKey(resourceModel.getResourceKey());
            sysLocaleResource.setResourceValue(resourceModel.getResourceValue());
            sysLocaleResource.setIsSystem(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());

            insertSysLocaleResource(sysLocaleResource);
            redisRes.deleteKey(resourceModel.getKey());
        }
    }

}
